@@ -1,19 +1,22 @@
 from __future__ import annotations
 
 import datetime as dt
+from contextlib import contextmanager
 from dataclasses import asdict, is_dataclass
 from pathlib import Path
 from tempfile import NamedTemporaryFile, TemporaryDirectory
-from typing import Any, Optional, Union
+from typing import Any, Iterator, Optional, Union
+from urllib.parse import urlparse
 
 import dbt.flags as flags
+from dbt.contracts.results import RunExecutionResult, RunResult, agate
+from dbt.logger import log_manager
+from dbt.main import initialize_config_values, parse_args, track_run
+
 from airflow import AirflowException
 from airflow.models.baseoperator import BaseOperator
 from airflow.models.xcom import XCOM_RETURN_KEY
 from airflow.utils.decorators import apply_defaults
-from dbt.contracts.results import RunExecutionResult, RunResult, agate
-from dbt.logger import log_manager
-from dbt.main import initialize_config_values, parse_args, track_run
 
 
 class DbtBaseOperator(BaseOperator):
@@ -53,6 +56,7 @@ class DbtBaseOperator(BaseOperator):
     @apply_defaults
     def __init__(
         self,
+        positional_args: Optional[list[str]] = None,
         project_dir: Optional[Union[str, Path]] = None,
         profiles_dir: Optional[Union[str, Path]] = None,
         profile: Optional[str] = None,
@@ -60,9 +64,11 @@ class DbtBaseOperator(BaseOperator):
         vars: Optional[dict[str, str]] = None,
         log_cache_events: Optional[bool] = False,
         bypass_cache: Optional[bool] = False,
+        s3_conn_id: str = "aws_default",
         **kwargs,
     ) -> None:
         super().__init__(**kwargs)
+        self.positional_args = positional_args
         self.project_dir = project_dir
         self.profiles_dir = profiles_dir
         self.profile = profile
@@ -70,32 +76,17 @@ class DbtBaseOperator(BaseOperator):
         self.vars = vars
         self.log_cache_events = log_cache_events
         self.bypass_cache = bypass_cache
+        self.s3_conn_id = s3_conn_id
+        self._dbt_s3_hook = None
 
     def execute(self, context: dict):
         """Execute dbt command with prepared arguments"""
-        if self.command is None:
-            raise AirflowException("dbt command is not defined")
+        with self.dbt_directory() as dbt_dir:  # type: str
+            with self.override_dbt_logging(dbt_dir):
+                args = self.prepare_args()
+                self.log.info("Running dbt %s with args %s", args[0], args[1:])
 
-        args: list[Optional[str]] = [self.command]
-        args.extend(self.args_list())
-
-        self.log.info("Running dbt %s with args %s", args[0], args[1:])
-
-        with TemporaryDirectory(prefix="airflowtmp") as tmp_dir:
-            with NamedTemporaryFile(dir=tmp_dir, mode="w+") as f:
-                with log_manager.applicationbound():
-                    log_manager.reset_handlers()
-                    log_manager.set_path(tmp_dir)
-                    # dbt logger writes to STDOUT and I haven't found a way
-                    # to bubble up to the Airflow command logger. As a workaround,
-                    # I set the output stream to a temporary file that is later
-                    # read and logged using the command's logger.
-                    log_manager.set_output_stream(f)
-                    res, success = self.run_dbt_command(args)
-
-                with open(f.name) as read_file:
-                    for line in read_file:
-                        self.log.info(line.rstrip())
+                res, success = self.run_dbt_command(args)
 
         if self.do_xcom_push is True:
             # Some dbt operations use dataclasses for its results,
@@ -110,6 +101,22 @@ class DbtBaseOperator(BaseOperator):
             raise AirflowException(f"dbt {args[0]} {args[1:]} failed")
         return res
 
+    def prepare_args(self) -> list[Optional[str]]:
+        """Prepare the arguments needed to call dbt"""
+        if self.command is None:
+            raise AirflowException(
+                "dbt command is not defined; each subclass of DbtBaseOperator"
+                "should set a command attribute"
+            )
+
+        args: list[Optional[str]] = [self.command]
+
+        if self.positional_args is not None:
+            args.extend(self.positional_args)
+        args.extend(self.args_list())
+
+        return args
+
     def args_list(self) -> list[str]:
         """Build a list of arguments to pass to dbt"""
         args = []
@@ -134,13 +141,77 @@ class DbtBaseOperator(BaseOperator):
             elif isinstance(value, dict):
                 yaml_str = (
                     "{"
-                    + ",".join("{}: {}".format(k, v) for k, v in value.items())
+                    + ", ".join("{}: {}".format(k, v) for k, v in value.items())
                     + "}"
                 )
                 args.append(yaml_str)
 
         return args
 
+    @contextmanager
+    def dbt_directory(self) -> Iterator[str]:
+        """
+        Creates a temporary directory for dbt to run in and prepares the dbt files
+        if they need to be pulled from S3.
+
+        Yields:
+        The temporary directory's name.
+        """
+        store_profiles_dir = self.profiles_dir
+        store_project_dir = self.project_dir
+
+        with TemporaryDirectory(prefix="airflowtmp") as tmp_dir:
+            if urlparse(str(self.profiles_dir)).scheme == "s3":
+                self.log.info("Fetching profiles.yml from S3")
+                profiles_file_path = self.dbt_s3_hook.get_dbt_profiles(
+                    self.profiles_dir,
+                    tmp_dir,
+                )
+                self.profiles_dir = str(profiles_file_path.parent) + "/"
+
+            if urlparse(str(self.project_dir)).scheme == "s3":
+                self.log.info("Fetching dbt project from S3")
+                project_dir_path = self.dbt_s3_hook.get_dbt_project(
+                    self.project_dir,
+                    tmp_dir,
+                )
+                self.project_dir = str(project_dir_path) + "/"
+
+            yield tmp_dir
+
+        self.profiles_dir = store_profiles_dir
+        self.project_dir = store_project_dir
+
+    @property
+    def dbt_s3_hook(self):
+        """Provides an existing DbtS3Hook or creates one"""
+        if self._dbt_s3_hook is None:
+            from airflow_dbt_python.hooks.dbt_s3 import DbtS3Hook
+
+            self._dbt_s3_hook = DbtS3Hook(self.s3_conn_id)
+        return self._dbt_s3_hook
+
+    @contextmanager
+    def override_dbt_logging(self, dbt_directory: str = None):
+        """
+        dbt logger writes to STDOUT and I haven't found a way
+        to bubble up to the Airflow command logger. As a workaround,
+        I set the output stream to a temporary file that is later
+        read and logged using the command's logger.
+        """
+        with NamedTemporaryFile(dir=dbt_directory, mode="w+") as f:
+            with log_manager.applicationbound():
+
+                log_manager.reset_handlers()
+                log_manager.set_path(dbt_directory)
+                log_manager.set_output_stream(f)
+
+                yield
+
+            with open(f.name) as read_file:
+                for line in read_file:
+                    self.log.info(line.rstrip())
+
     def run_dbt_command(self, args: list[Optional[str]]) -> tuple[RunResult, bool]:
         """Run a dbt command as implemented by a subclass"""
         try:
@@ -440,6 +511,71 @@ class DbtLsOperator(DbtBaseOperator):
 DbtListOperator = DbtLsOperator
 
 
+class DbtRunOperationOperator(DbtBaseOperator):
+    """Execute dbt run-operation"""
+
+    command = "run-operation"
+
+    __dbt_args__ = DbtBaseOperator.__dbt_args__ + [
+        "args",
+    ]
+
+    def __init__(
+        self,
+        macro: str,
+        args: Optional[dict[str, str]] = None,
+        **kwargs,
+    ) -> None:
+        super().__init__(positional_args=[macro], **kwargs)
+        self.args = args
+
+
+class DbtParseOperator(DbtBaseOperator):
+    """Execute dbt parse"""
+
+    command = "parse"
+
+    def __init__(
+        self,
+        **kwargs,
+    ) -> None:
+        super().__init__(**kwargs)
+
+
+class DbtSourceOperator(DbtBaseOperator):
+    """Execute dbt source"""
+
+    command = "source"
+
+    __dbt_args__ = DbtBaseOperator.__dbt_args__ + [
+        "select",
+        "models",
+        "exclude",
+        "selector",
+        "dbt_output",
+    ]
+
+    def __init__(
+        self,
+        # Only one subcommand is currently provided
+        subcommand: str = "snapshot-freshness",
+        select: Optional[list[str]] = None,
+        dbt_output: Optional[Union[str, Path]] = None,
+        threads: Optional[int] = None,
+        exclude: Optional[list[str]] = None,
+        selector: Optional[str] = None,
+        state: Optional[Union[str, Path]] = None,
+        **kwargs,
+    ) -> None:
+        super().__init__(positional_args=[subcommand], **kwargs)
+        self.select = select
+        self.dbt_output = dbt_output
+        self.threads = threads
+        self.exclude = exclude
+        self.selector = selector
+        self.state = state
+
+
 def run_result_factory(data: list[tuple[Any, Any]]):
     """
     We need to handle dt.datetime and agate.table.Table.
